use std::fmt;
use std::str::FromStr;

use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
pub struct CUSA {
    // sa id
    pub id: String,

    // sa's url
    pub url: String,

    // sa title
    pub title: String,

    // the major severity
    pub severity: Severity,

    pub description: String,

    // 包含的 cve 列表
    pub cves: Vec<CVE>,
}

impl CUSA {
    /// sa id
    pub fn id(&self) -> &str {
        &self.id
    }

    /// sa info url
    pub fn url(&self) -> &str {
        &self.url
    }

    /// title
    pub fn title(&self) -> &str {
        &self.title
    }

    /// severity lvl
    pub fn severity(&self) -> &Severity {
        &self.severity
    }

    /// 详尽的描述信息
    pub fn description(&self) -> &str {
        &self.description
    }

    /// 包含的 cve 列表
    pub fn cves(&self) -> &Vec<CVE> {
        &self.cves
    }
}

#[derive(Debug, Clone, PartialOrd, Ord, PartialEq, Eq, Hash, Serialize, Deserialize)]
pub enum Severity {
    None,
    Low,
    Medium,   // Moderate
    High,     // Important
    Critical,
}

impl Severity {
    pub fn new() -> Self {
        Severity::None
    }
}

// 为枚举 Severity 实现 FromStr trait
impl FromStr for Severity {
    type Err = ParseSeverityError;

    // bug fix: 华为不干人事儿，单词都能拼错，现在只取首字母
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s.to_lowercase().chars().next() {
            Some('n') => Ok(Severity::None),
            Some('l') => Ok(Severity::Low),
            Some('m') => Ok(Severity::Medium),
            Some('i') | Some('h') => Ok(Severity::High),
            Some('c') => Ok(Severity::Critical),
            _ => Err(ParseSeverityError::InvalidSeverity),
        }
    }
}

impl From<&Severity> for usize {
    fn from(item: &Severity) -> usize {
        match *item {
            Severity::None => 0,
            Severity::Low => 1,
            Severity::Medium => 2,
            Severity::High => 3,
            Severity::Critical => 4,
        }
    }
}

// 定义 ParseSeverityError 枚举类型来表示解析错误
#[derive(Debug, Clone)]
pub enum ParseSeverityError {
    InvalidSeverity,
}

// 为 ParseSeverityError 实现 Display trait，以便更好地显示错误信息
impl fmt::Display for ParseSeverityError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            ParseSeverityError::InvalidSeverity => write!(f, "Invalid severity level"),
        }
    }
}

// 为 ParseSeverityError 实现 std::error::Error trait
impl std::error::Error for ParseSeverityError {}

#[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
pub struct CVE {
    // cve id
    pub id: String,

    // cve 官网地址
    pub url: String,

    // 严重级别
    pub severity: Severity,
}
